package com.hz.pm.api.meeting.task;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.util.StrUtils;
import com.hz.pm.api.external.sms.SmsServiceClient;
import com.hz.pm.api.external.sms.dto.SmsDTO;
import com.hz.pm.api.external.sms.vo.SmsReply;
import com.hz.pm.api.external.sms.vo.SmsReplyResponse;
import com.hz.pm.api.external.sms.vo.SmsSendResponse;
import com.hz.pm.api.meeting.entity.domain.ExpertInviteRule;
import com.hz.pm.api.meeting.entity.domain.Meeting;
import com.hz.pm.api.meeting.entity.domain.MeetingExpert;
import com.hz.pm.api.meeting.entity.domain.MeetingExpertSms;
import com.hz.pm.api.meeting.entity.dto.RandomInviteRuleDTO;
import com.hz.pm.api.meeting.entity.dto.SmsReplyDetails;
import com.hz.pm.api.meeting.entity.dto.YxtCallBackDTO;
import com.hz.pm.api.meeting.entity.enumeration.ExpertAttendStatusEnum;
import com.hz.pm.api.meeting.entity.enumeration.ExpertInviteTypeEnum;
import com.hz.pm.api.meeting.mapper.MeetingExpertSmsMapper;
import com.hz.pm.api.meeting.service.IExpertInviteRuleService;
import com.hz.pm.api.meeting.service.IMeetingExpertService;
import com.hz.pm.api.sms.constant.VoiceSmsTemplateConst;
import com.ningdatech.basic.util.StrPool;
import com.ningdatech.yxt.entity.SysMsgRecordDetail;
import com.ningdatech.yxt.service.ISysMsgRecordDetailService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.time.Duration;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;

import static com.hz.pm.api.meeting.entity.enumeration.ExpertAttendStatusEnum.*;

/**
 * <p>
 * ExpertCallResultRewriteTask
 * <br>
 * 专家电话结果回填
 * </p>
 *
 * @author WendyYang
 * @since 14:15 2022/8/18
 */
@Slf4j
@Component
@AllArgsConstructor
public class ExpertCallResultRewriteTask {

    private final RandomInviteProperties randomInviteProperties;

    private final ThreadPoolTaskScheduler scheduler;
    private final IMeetingExpertService meetingExpertService;
    private final IExpertInviteRuleService inviteRuleService;
    private final ISysMsgRecordDetailService msgRecordDetailService;
    private final SmsServiceClient smsServiceClient;
    private final MeetingExpertSmsMapper meetingExpertSmsMapper;

    private final static int MINUTES_CALL_RESULT_FEEDBACK = 15;

    private static final String AGREE_KEY = "1";

    @PostConstruct
    public void initTask() {
        if (!randomInviteProperties.getEnable()) {
            log.warn("随机邀请已关闭……");
            return;
        }
        // 处理电话结果回填
        Instant startTime = Instant.now().plus(randomInviteProperties.getResultRewriteFixedRate(), ChronoUnit.MINUTES);
        Duration fixedRate = Duration.ofMinutes(randomInviteProperties.getResultRewriteFixedRate());
//        scheduler.scheduleAtFixedRate(this::rewritePhoneCallResult, startTime, fixedRate);

        // 处理短信结果回填
        Instant smsStartTime = Instant.now().plus(randomInviteProperties.getResultRewriteFixedRate(), ChronoUnit.MINUTES);
        scheduler.scheduleAtFixedRate(this::expertSmsReply, smsStartTime, fixedRate);
    }

    public void expertSmsReply() {
        log.info("开始执行短信回复查询任务：{}", Thread.currentThread().getName());
        // 查询所有邀请的专家信息 状态为通话中的
        LambdaQueryWrapper<MeetingExpert> meQuery = Wrappers.lambdaQuery(MeetingExpert.class)
                .eq(MeetingExpert::getStatus, NOTICING.getCode())
                .eq(MeetingExpert::getInviteType, ExpertInviteTypeEnum.RANDOM.getCode());
        List<MeetingExpert> experts = meetingExpertService.list(meQuery);
        if (experts.isEmpty()) {
            log.info("暂无短信回复任务执行");
            return;
        }
        // 所有随机邀请的短信Uuid
        List<String> smsUuids = experts.stream()
                .map(expert -> expert.getSmsUuid())
                .filter(uuid -> uuid != null)
                .distinct()
                .collect(Collectors.toList());
        Set<Long> randomRuleIds = experts.stream()
                .map(MeetingExpert::getRuleId).collect(Collectors.toSet());
        // 查询随机邀请回调等待时间
        Map<Long, Integer> callbackMinutes = new HashMap<>(randomRuleIds.size());
        if (!randomRuleIds.isEmpty()) {
            List<ExpertInviteRule> inviteRules = inviteRuleService.listByIds(randomRuleIds);
            inviteRules.forEach(expert -> {
                RandomInviteRuleDTO rule = JSONObject.parseObject(expert.getInviteRule(), RandomInviteRuleDTO.class);
                callbackMinutes.put(expert.getId(), rule.getWaitForCallbackMinutes());
            });
        }

        // 获取专家回复内容
        SmsReplyDetails smsReplyDetails = viewSmsReplies(smsUuids);

        List<MeetingExpert> updates = new ArrayList<>();

        for (MeetingExpert expert : experts) {

            Integer minutes = ObjectUtil.defaultIfNull(callbackMinutes.get(expert.getRuleId()), MINUTES_CALL_RESULT_FEEDBACK);
            // 判断回复状态
            Optional<Integer> status = getStatusByMsgRecordDetail(smsReplyDetails, minutes, expert);
            if (status.isPresent()) {
                MeetingExpert update = new MeetingExpert();
                update.setUpdateBy(0L);
                update.setUpdateOn(LocalDateTime.now());
                update.setId(expert.getId());
                update.setStatus(status.get());
                updates.add(update);
            }
        }
        meetingExpertService.updateBatchById(updates);
    }


    public void rewritePhoneCallResult() {
        log.info("开始执行电话结果回填任务：{}", Thread.currentThread().getName());
        // 查询所有邀请的专家信息 状态为通话中的
        LambdaQueryWrapper<MeetingExpert> meQuery = Wrappers.lambdaQuery(MeetingExpert.class)
                .eq(MeetingExpert::getStatus, NOTICING.getCode())
                .eq(MeetingExpert::getInviteType, ExpertInviteTypeEnum.RANDOM.getCode());
        List<MeetingExpert> experts = meetingExpertService.list(meQuery);
        if (experts.isEmpty()) {
            log.info("暂无电话结果回填任务执行");
            return;
        }
        // 所有随机邀请的规则ID
        Map<Long, String> submitKeys = new HashMap<>(experts.size());
        Set<Long> randomRuleIds = experts.stream().peek(w -> submitKeys.put(w.getId(), w.getSubmitKey()))
                .map(MeetingExpert::getRuleId).collect(Collectors.toSet());
        // 查询随机邀请回调等待时间
        Map<Long, Integer> callbackMinutes = new HashMap<>(randomRuleIds.size());
        if (!randomRuleIds.isEmpty()) {
            List<ExpertInviteRule> inviteRules = inviteRuleService.listByIds(randomRuleIds);
            inviteRules.forEach(w -> {
                RandomInviteRuleDTO rule = JSONObject.parseObject(w.getInviteRule(), RandomInviteRuleDTO.class);
                callbackMinutes.put(w.getId(), rule.getWaitForCallbackMinutes());
            });
        }
        LambdaQueryWrapper<SysMsgRecordDetail> msgRecordDetailQuery = Wrappers.lambdaQuery(SysMsgRecordDetail.class)
                .in(SysMsgRecordDetail::getSubmitKey, submitKeys.values());
        List<SysMsgRecordDetail> recordDetailList = msgRecordDetailService.list(msgRecordDetailQuery);
        if (recordDetailList.isEmpty()) {
            return;
        }
        Map<String, SysMsgRecordDetail> recordDetailMap = recordDetailList.stream()
                .collect(Collectors.toMap(w -> w.getSubmitKey() + StrPool.UNDERSCORE + w.getReceiveNumber(), w -> w));
        List<MeetingExpert> updates = new ArrayList<>(), agrees = new ArrayList<>();
        for (MeetingExpert expert : experts) {
            String key = expert.getSubmitKey() + StrPool.UNDERSCORE + expert.getMobile();
            SysMsgRecordDetail msgRecordDetail = recordDetailMap.get(key);
            if (msgRecordDetail == null) {
                // 极端情况下获取不到submitKey异常情况
                continue;
            }
            Integer minutes = ObjectUtil.defaultIfNull(callbackMinutes.get(expert.getRuleId()), MINUTES_CALL_RESULT_FEEDBACK);
            Optional<Integer> status = getStatusByMsgRecordDetail(msgRecordDetail, minutes, expert.getCreateOn());
            if (status.isPresent()) {
                MeetingExpert update = new MeetingExpert();
                update.setUpdateBy(0L);
                update.setUpdateOn(LocalDateTime.now());
                update.setId(expert.getId());
                update.setStatus(status.get());
                if (AGREED.eq(update.getStatus())) {
                    // 发送专家确认参加的短信通知
                    // sendAgreeNotice(expert);
                    agrees.add(expert);
                }
                updates.add(update);
            }
        }
        meetingExpertService.updateBatchById(updates);
    }

    private static Optional<Integer> getStatusByMsgRecordDetail(SysMsgRecordDetail mrd, int minutes, LocalDateTime createOn) {
        LocalDateTime limitTime = LocalDateTime.now().minusMinutes(minutes);
        String callBackJson = mrd.getCallBackJson();
        boolean waiting = limitTime.isBefore(createOn);
        boolean hasCallBack = StrUtils.isNotBlank(callBackJson);
        if (!hasCallBack && waiting) {
            return Optional.empty();
        }
        ExpertAttendStatusEnum status;
        if (hasCallBack) {
            try {
                YxtCallBackDTO callback = JSONObject.parseObject(callBackJson, YxtCallBackDTO.class);
                LocalDateTime dialBeginTime = callback.getDialBeginTime();
                if (dialBeginTime == null) {
                    return Optional.empty();
                }
                Integer resultCode = callback.getResultCode();
                if (resultCode != null && resultCode == 0) {
                    String pressKey = callback.getPressKey();
                    if (pressKey != null) {
                        pressKey = pressKey.replaceAll("\\*", "").trim();
                    }
                    if (StrUtils.isBlank(pressKey) && waiting) {
                        return Optional.empty();
                    }
                    status = AGREE_KEY.equals(pressKey) ? AGREED : REFUSED;
                } else {
                    if (waiting) {
                        return Optional.empty();
                    }
                    status = REFUSED;
                }
            } catch (Exception e) {
                log.error("获取电话回调结果异常{}", mrd, e);
                status = UNANSWERED;
            }
        } else {
            // 超时未回复设置为拒绝参加
            status = REFUSED;
        }
        return Optional.of(status.getCode());
    }

    private static Optional<Integer> getStatusByMsgRecordDetail(SmsReplyDetails smsReplyDetails, int minutes, MeetingExpert expert) {
        Map<String, List<SmsReply>> accept = smsReplyDetails.getAccept() //回复1同意参加的手机号
                , reject = smsReplyDetails.getReject() //回复2拒绝参加的手机号
                , other = smsReplyDetails.getOther(); //回复其他的专家
        Map<String, List<String>> errorPhones = new HashMap<>();//发送失败的专家
        // 专家最长响应时间
        LocalDateTime limitTime = LocalDateTime.now().minusMinutes(minutes);
        // 专家抽取时间
        boolean waiting = limitTime.isBefore(expert.getCreateOn());
        ExpertAttendStatusEnum status = NOTICING;
        boolean hasCallBack = ObjectUtil.isEmpty(smsReplyDetails);
        if (hasCallBack && waiting) {
            return Optional.empty();
        }
//        if (!waiting) {
//            status = REFUSED;
//        }
        List<String> errorPhoneList = errorPhones.get(expert.getSmsUuid());
        if(ObjectUtil.isNotEmpty(errorPhoneList) && errorPhoneList.contains(expert.getMobile())){
            // 未应答
            status = UNANSWERED;
        }
        if(ObjectUtil.isNotEmpty(accept)) {
            List<SmsReply> smsRepliesAccept = accept.get(expert.getSmsUuid());
            if (ObjectUtil.isNotEmpty(smsRepliesAccept)) {
                boolean containsReplyMobile = smsRepliesAccept.stream()
                        .anyMatch(reply -> reply.getReplyMobile().equals(expert.getMobile()));
                if (containsReplyMobile) {
                    SmsReply filteredReplies = smsRepliesAccept.stream()
                            .filter(reply -> reply.getReplyMobile().equals(expert.getMobile()))
                            .collect(Collectors.toList()).get(0);
                    // 回复时间
                    Instant instant = Instant.ofEpochMilli(filteredReplies.getReplyReplytime());
                    LocalDateTime localDateTime = LocalDateTime.ofInstant(instant, ZoneId.systemDefault());
                    if (localDateTime.isBefore(expert.getCreateOn().plusMinutes(minutes))) {
                        status = AGREED;
                    } else {
                        status = REFUSED;
                    }
                }
            }
        }
        if(ObjectUtil.isNotEmpty(reject)) {
            List<SmsReply> smsRepliesReject = reject.get(expert.getSmsUuid());
            if (ObjectUtil.isNotEmpty(smsRepliesReject)) {
                boolean containsReplyMobile = smsRepliesReject.stream()
                        .anyMatch(reply -> reply.getReplyMobile().equals(expert.getMobile()));
                if (containsReplyMobile) {
                    status = REFUSED;
                }
            }
        }
        if(ObjectUtil.isNotEmpty(other)) {
            List<SmsReply> smsRepliesOther = other.get(expert.getSmsUuid());
            if (ObjectUtil.isNotEmpty(smsRepliesOther)) {
                boolean containsReplyMobile = smsRepliesOther.stream()
                        .anyMatch(reply -> reply.getReplyMobile().equals(expert.getMobile()));
                if (containsReplyMobile) {
                    status = REFUSED;
                }
            }
        }
        return Optional.of(status.getCode());
    }

    // 发送短信提醒
    public String sendExpertSms(List<MeetingExpert> expertMeetings, Meeting meeting) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        // 短信内容
        String replacedContent = String.format(VoiceSmsTemplateConst.EXPERT_INVITE,
                meeting.getCreator(), meeting.getName(), meeting.getStartTime().format(formatter) + "至" + meeting.getEndTime().format(formatter), meeting.getMeetingAddress());

        List<String> phones = expertMeetings.stream().map(obj -> obj.getMobile()).collect(Collectors.toList());
        SmsDTO<SmsSendResponse> smsSendResponseSmsDto = smsServiceClient.smsSend(replacedContent, phones);
        // 短信发送成功返回的UUID
        String resultUuid = smsSendResponseSmsDto.getData().getResult();
        // 保存短信发送记录
        MeetingExpertSms meetingExpertSms = MeetingExpertSms.builder()
                .meetingId(meeting.getId().toString())
                .createOn(LocalDateTime.now())
                .smsUuid(resultUuid)
                .content(replacedContent)
                .phones(phones.stream()
                        .collect(Collectors.joining(", ")))
                .smsResult(smsSendResponseSmsDto.toString())
                .build();
        meetingExpertSmsMapper.insert(meetingExpertSms);
        return resultUuid;
    }

    // 查看短信回复内容并更改专家回复状态
    public SmsReplyDetails viewSmsReplies(List<String> uuids) {
        Map<String, List<SmsReply>> accept = new HashMap<>()
                , reject = new HashMap<>()
                , other = new HashMap<>();
        Map<String, List<String>> errorPhones = new HashMap<>();
        uuids.forEach(uuid -> {
            SmsReplyResponse response = smsServiceClient.smsReply(uuid);
            List<SmsReply> smsReplies = response.getResult();
            // 成功回复的手机信息
            if (ObjectUtil.isNotEmpty(smsReplies)){
                // 只有第一条回复的内容有效
                smsReplies = smsReplies.stream()
                        .collect(Collectors.groupingBy(SmsReply::getReplyMobile,
                                Collectors.minBy(Comparator.comparing(SmsReply::getReplyReplytime))))
                        .values().stream()
                        .map(Optional::get)
                        .collect(Collectors.toList());
                accept.put(uuid, smsReplies.stream()
                        .filter(obj -> ObjectUtil.isNotEmpty(obj))
                        .filter(smsReply -> "1".equals(smsReply.getReplyContent()))
                        .collect(Collectors.toList()));
                reject.put(uuid, smsReplies.stream()
                        .filter(obj -> ObjectUtil.isNotEmpty(obj))
                        .filter(smsReply -> "2".equals(smsReply.getReplyContent()))
                        .collect(Collectors.toList()));
                other.put(uuid, smsReplies.stream()
                        .filter(obj -> ObjectUtil.isNotEmpty(obj))
                        .filter(smsReply -> !("1".equals(smsReply.getReplyContent()) || "2".equals(smsReply.getReplyContent())))
                        .collect(Collectors.toList()));
            }
            // 发送失败的手机号
            if (StrUtils.isNotBlank(response.getErrorPhone())){
                errorPhones.put(uuid, Arrays.asList(response.getErrorPhone().split(",")));
            }
        });

        SmsReplyDetails smsReplyDetails = SmsReplyDetails.builder()
                .accept(accept)
                .reject(reject)
                .errorPhones(errorPhones).build();

        return smsReplyDetails;
    }
}
